{% load i18n static %}


<html lang="en">
<head>
  <meta charset="UTF-8">
  <meta http-equiv="X-UA-Compatible" content="IE=edge">
  <meta name="viewport" content="width=device-width, initial-scale=1.0">
  <title>Document</title>
  
  {% block extrastyle %}
    <link rel="stylesheet" type="text/css" href="{% static "admin/css/login.css" %}">
    {{ form.media }}
  {% endblock %}
</head>
<body>

  
{% block content %}
  {% if form.errors and not form.non_field_errors %}
    <p class="errornote">
    {% if form.errors.items|length == 1 %}{% translate "Please correct the error below." %}{% else %}{% translate "Please correct the errors below." %}{% endif %}
    </p>
  {% endif %}

  {% if form.non_field_errors %}
    {% for error in form.non_field_errors %}
    <p class="errornote">
        {{ error }}
    </p>
    {% endfor %}
  {% endif %}

  <div id="content-main">

  {% if user.is_authenticated %}
    <p class="errornote">
    {% blocktranslate trimmed %}
        You are authenticated as {{ username }}, but are not authorized to
        access this page. Would you like to login to a different account?
    {% endblocktranslate %}
    </p>
  {% endif %}

  <form action="{{ app_path }}" method="post" id="login-form">{% csrf_token %}
    <div class="form-row">
      {{ form.username.errors }}
      {{ form.username.label_tag }} {{ form.username }}
    </div>
    <div class="form-row">
      {{ form.password.errors }}
      {{ form.password.label_tag }} {{ form.password }}
      <input type="hidden" name="next" value="{{ next }}">
    </div>
    {% url 'admin_password_reset' as password_reset_url %}
    {% if password_reset_url %}
    <div class="password-reset-link">
      <a href="{{ password_reset_url }}">{% translate 'Forgotten your password or username?' %}</a>
    </div>
    {% endif %}
    <div class="submit-row">
      <input type="submit" value="{% translate 'Log in' %}">
    </div>
  </form>

  </div>
{% endblock %}

</body>
</html>
